<?php

if (!defined('ABSPATH')) {
    exit;
}

class IOTPay_API
{

    private static $app_id = '';
    private static $app_secret = '';
    private static $app_username = '';

    /**
     * Set app_id.
     * @param string
     */
    public static function set_app_id($app_id)
    {
        self::$app_id = $app_id;
    }

    /**
     * Get app_id.
     * @return string
     */
    public static function get_app_id()
    {
        if (!self::$app_id) {
            $options = get_option('woocommerce_iotpay_wechatpay_settings');
            if (isset($options['app_id'])) {
                self::set_app_id($options['app_id']);
            }
        }
        return self::$app_id;
    }

    /**
     * Set app_secret.
     * @param string
     */
    public static function set_app_secret($app_secret)
    {
        self::$app_secret = $app_secret;
    }

    /**
     * Get app_secret.
     * @return string
     */
    public static function get_app_secret()
    {
        if (!self::$app_secret) {
            $options = get_option('woocommerce_iotpay_wechatpay_settings');
            if (isset($options['app_secret'])) {
                self::set_app_secret($options['app_secret']);
            }
        }
        return self::$app_secret;
    }

    public static function set_app_username($app_username)
    {
        self::$app_username = $app_username;
    }

    public static function get_app_username()
    {
        if (!self::$app_username) {
            $options = get_option('woocommerce_iotpay_wechatpay_settings');
            if (isset($options['app_username'])) {
                self::set_app_username($options['app_username']);
            }
        }
        return self::$app_username;
    }

    public static function generate_iotpay_order($order, $channel)
    {
        $order_id = method_exists($order, 'get_id') ? $order->get_id() : $order->id;
        $new_order_id = date_i18n("ymdHis") . rand(1000, 9999) . $order_id;
        update_post_meta($order_id, 'mchOrderNo', $new_order_id);
        update_post_meta($order_id, 'channel', $channel);
        $price = (int)($order->get_total() * 100);
        if ($price < 1) {
            throw new Exception('The payment amount is too little!');
        }
        $merchant_id = self::get_app_id();
        $merchant_key = self::get_app_secret();
        $username = self::get_app_username();
        $notify_url = IOTPAY_URL . '/notify.php';
        if ($channel == 'CreditCard') {
            $url = 'https://api.iotpaycloud.com/v2/cc_purchase';
            $arr = array(
                'mchId' => $merchant_id,
                'mchOrderNo' => $new_order_id,
                'customerId' => sprintf("%08d", rand(100, 999)),
                'currency' => 'CAD',
                'amount' => $price,
                'jobNo' => $username,
                'notifyUrl' => $notify_url,
                'returnUrl' => IOTPAY_URL . '/return.php?order_id=' . $order_id,
                'subject' => str_replace('&', ' ', self::get_order_title($order)),
                'body' => str_replace('&', ' ', self::get_order_title($order)),
                'source' => 'wordpress'
            );
        } else if ($channel == 'WX_JSAPI') {
            $url = 'https://api.iotpaycloud.com/v1/payForSubmit';
            $arr = array(
                'mchId' => $merchant_id,
                'mchOrderNo' => $new_order_id,
                'channelId' => 'WX_JSAPI',
                'currency' => 'CAD',
                'amount' => $price,
                'clientIp' => self::real_ip(),
                'notifyUrl' => $notify_url,
                'returnUrl' => IOTPAY_URL . '/return.php?order_id=' . $order_id,
                'subject' => str_replace('&', ' ', self::get_order_title($order)),
                'body' => str_replace('&', ' ', self::get_order_title($order)),
                'jobNo' => $username,
                'source' => 'wordpress',
                'qrcode_pay' => '1'
            );
        } else {
            $url = 'https://api.iotpaycloud.com/v1/create_order';
            if ($channel == 'WX_NATIVE') {
                $sceneInfo = '{"productId":"' . $new_order_id . '"}';
            } else {
                $sceneInfo = '';
            }
            $arr = array(
                'mchId' => $merchant_id,
                'mchOrderNo' => $new_order_id,
                'extra' => $sceneInfo,
                'channelId' => $channel,
                'currency' => 'CAD',
                'amount' => $price,
                'clientIp' => self::real_ip(),
                'device' => 'WEB',
                'notifyUrl' => $notify_url,
                'returnUrl' => IOTPAY_URL . '/return.php?order_id=' . $order_id,
                'subject' => str_replace('&', ' ', self::get_order_title($order)),
                'body' => str_replace('&', ' ', self::get_order_title($order)),
                'jobNo' => $username,
                'source' => 'wordpress'
            );
        }
        $sort_array = self::arg_sort($arr);
        $arr['sign'] = self::build_mysign($sort_array, $merchant_key, "MD5");
        if ($channel == 'CreditCard') {
            $param = json_encode($arr);
        } else {
            $param = 'params=' . json_encode($arr);
        }
        file_put_contents('./wp-content/plugins/iotpay-wechatpay-alipay-for-woocommerce/log.txt', $param . "\r\n", FILE_APPEND);
        $args = array(
            'body' => $param,
            'timeout' => '30',
            'redirection' => '5',
            'httpversion' => '1.0',
            'blocking' => true,
            'headers' => array(),
            'cookies' => array()
        );
        $response = wp_remote_post($url, $args);
        $result = $response['body'];
        file_put_contents('./wp-content/plugins/iotpay-wechatpay-alipay-for-woocommerce/log.txt', $result . "\r\n", FILE_APPEND);
        $resArr = json_decode($result, false);
        if (!$resArr) {
            throw new Exception('This request has been rejected by the IOTPay service!');
        }
        if ($channel == 'WX_JSAPI') {
            if (!isset($resArr->success) || $resArr->success != true) {
                throw new Exception(sprintf('ERROR MSG:%s.', $resArr->message));
            }
        } else {
            if (!isset($resArr->retCode) || $resArr->retCode != 'SUCCESS') {
                throw new Exception(sprintf('ERROR MSG:%s.', $resArr->retMsg));
            }
        }
        if ($channel == 'ALIPAY_PC' || $channel == 'ALIPAY_WAP') {
            header("Location:" . $resArr->retParams->alipayUrl);
        } elseif ($channel == 'CreditCard') {
            header("Location:" . $resArr->payUrl);
        } elseif ($channel == 'UPI_SEQ') {
            header("Location:" .  $resArr->retParams->unionpayUrl);
        } else {
            return $resArr;
        }
    }

    public static function get_order_title($order, $limit = 32, $trimmarker = '...')
    {
        $title = "";
        $order_items = $order->get_items();
        if ($order_items) {
            $qty = count($order_items);
            foreach ($order_items as $item_id => $item) {
                $title .= "{$item['name']}";
                break;
            }
            if ($qty > 1) {
                $title .= '...';
            }
        }
        $title = mb_strimwidth($title, 0, $limit, 'utf-8');
        return apply_filters('payment-get-order-title', $title, $order);
    }

    public static function real_ip()
    {
        static $realip = NULL;
        if ($realip !== NULL) {
            return $realip;
        }
        if (isset($_SERVER)) {
            if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
                $arr = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
                /* 取X-Forwarded-For中第一个非unknown的有效IP字符串 */
                foreach ($arr as $ip) {
                    $ip = trim($ip);
                    if ($ip != 'unknown') {
                        $realip = $ip;
                        break;
                    }
                }
            } elseif (isset($_SERVER['HTTP_CLIENT_IP'])) {
                $realip = $_SERVER['HTTP_CLIENT_IP'];
            } else {
                if (isset($_SERVER['REMOTE_ADDR'])) {
                    $realip = $_SERVER['REMOTE_ADDR'];
                } else {
                    $realip = '0.0.0.0';
                }
            }
        } else {
            if (getenv('HTTP_X_FORWARDED_FOR')) {
                $realip = getenv('HTTP_X_FORWARDED_FOR');
            } elseif (getenv('HTTP_CLIENT_IP')) {
                $realip = getenv('HTTP_CLIENT_IP');
            } else {
                $realip = getenv('REMOTE_ADDR');
            }
        }
        preg_match("/[\d\.]{7,15}/", $realip, $onlineip);
        $realip = !empty($onlineip[0]) ? $onlineip[0] : '0.0.0.0';

        return $realip;
    }

    public static function arg_sort($array)
    {
        ksort($array);
        reset($array);
        return $array;
    }

    public static function md5sign($prestr, $sign_type)
    {
        $sign = '';
        if ($sign_type == 'MD5') {
            $sign = strtoupper(md5($prestr)); //全部大写
        } else {
            die("暂不支持" . $sign_type . "类型的签名方式");
        }
        return $sign;
    }

    public static function build_mysign($sort_array, $key, $sign_type = "MD5")
    {
        $prestr = self::create_linkstring($sort_array);
        $prestr = $prestr . "&key=" . $key;
        $mysgin = self::md5sign($prestr, $sign_type);
        return $mysgin;
    }

    public static function create_linkstring($array)
    {
        $arg = "";
        foreach ($array as $key => $val) {
            if ($val !== '') {
                $arg .= $key . "=" . $val . "&";
            }
        }
        $arg = substr($arg, 0, strlen($arg) - 1);             //去掉最后一个&字符
        return $arg;
    }

    public static function sign_verify($arr)
    {
        if (!$arr) {
            return "fail";
        }
        $sign = $arr['sign'];
        unset($arr["sign"]);
        $sort_array = self::arg_sort($arr);
        $mysign = self::build_mysign($sort_array, self::get_app_secret(), "MD5");
        if ($mysign == $sign) {
            return "success";
        } else {
            return "fail";
        }
    }

    public static function iotpay_notify()
    {
        parse_str($_SERVER['QUERY_STRING'], $arr);
        $res = self::sign_verify($arr);
        if ($res == "success") {
            $order_id = substr($arr["mchOrderNo"], 16);
            $order = new WC_Order($order_id);
            if ($order && $order->needs_payment()) {
                update_post_meta($order_id, 'iotpay_transaction_no', $arr["payOrderId"]);
                try {
                    $order->payment_complete($order_id);
                    if ($_SERVER['REQUEST_METHOD'] == "POST") {
                        echo "success";
                        exit;
                    } else {
                        header("Location: " . $order->get_checkout_order_received_url());
                    }
                } catch (Exception $e) {
                    exit;
                }
            }
        }

        if ($_SERVER['REQUEST_METHOD'] == "POST") {
            echo "success";
            exit;
        } else {
            header("Location: " . $order->get_checkout_order_received_url());
        }
    }
}
