<?php

if (!defined('ABSPATH')) {
    exit;
}

// Setup our Gateway's id, description and other values
class WC_IOTPay_WechatPay extends WC_Payment_Gateway
{

    public function __construct()
    {
        // The global ID for this Payment method
        $this->id = "iotpay_wechatpay";

        // The Title shown on the top of the Payment Gateways Page next to all the other Payment Gateways
        $this->method_title = 'IOTPay - Wechat Pay';

        // The description for this Payment Gateway, shown on the actual Payment options page on the backend
        $this->method_description = 'Alipay and Wechat Payment provided by <a href="https://www.iotpay.ca" target="_blank">IOTPay</a>';

        // If you want to show an image next to the gateway's name on the frontend, enter a URL to an image.
        $this->icon = IOTPAY_URL . '/assets/images/wechat-logo.png';

        $this->has_fields = false;

        // Supports
        $this->supports[] = 'refunds';

        // This basically defines your settings which are then loaded with init_settings()
        $this->init_form_fields();

        // After init_settings() is called, you can get the settings and load them into variables, e.g:
        $this->init_settings();

        // Turn these settings into variables we can use
        foreach ($this->settings as $setting_key => $value) {
            $this->$setting_key = $value;
        }

        if (is_admin()) {
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        }


        add_action("wp_ajax_iotpay_order_status", array($this, 'wc_order_is_paid'));
        add_action("wp_ajax_nopriv_iotpay_order_status", array($this, 'wc_order_is_paid'));

        add_action('wp_enqueue_scripts', array($this, 'my_admin_scripts'));
        add_action('woocommerce_api_iotpay_wechatpay_jsapi_return', array($this, 'jsapi_return'));
    }

    public function my_admin_scripts()
    {
        wp_register_style('iotpay-style', plugins_url('assets/css/iotpay-style.css', IOTPAY_FILE), array());
        wp_enqueue_style('iotpay-style');
    }

    public function get_icon()
    {
        $icons_str = '<img src="' . IOTPAY_URL . '/assets/images/wechat-logo.png" class="right-float" alt="Wechat Pay" />';
        return apply_filters('woocommerce_gateway_icon', $icons_str, $this->id);
    }


    public function is_available()
    {

        if (!$this->app_id || !$this->app_secret || !$this->app_username) {
            return false;
        }

        if ($this->enabled == 'no') {
            return false;
        }
        return $this->enabled;
    }


    function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'woocommerce'),
                'type' => 'checkbox',
                'label' => __('Enable', 'woocommerce'),
                'default' => 'no',
                'section' => 'default'
            ),
            'title' => array(
                'title' => __('Title', 'woocommerce'),
                'type' => 'text',
                'default' => 'Wechat Pay',
                'desc_tip' => true,
                'css' => 'width:400px',
                'section' => 'default'
            ),
            'description' => array(
                'title' => __('Description', 'woocommerce'),
                'type' => 'textarea',
                'default' => 'Use Wechat App to Scan QR Code to complete the payment on next page',
                'desc_tip' => true,
                'css' => 'width:400px',
                'section' => 'default'
            ),
            'app_id' => array(
                'title' => 'Merchant Id',
                'type' => 'text',
                'default' => '',
                'css' => 'width:400px'
            ),
            'app_secret' => array(
                'title' => 'Merchant Key',
                'type' => 'password',
                'default' => '',
                'css' => 'width:400px'
            ),
            'app_username' => array(
                'title' => 'Username',
                'type' => 'text',
                'default' => '',
                'css' => 'width:400px'
            )
        );
    }


    /**
     * Output for the order received page.
     */
    public function thankyou_page()
    {
        if ($this->instructions) {
            echo wpautop(wptexturize($this->instructions));
        }
    }

    public function is_wechat_client()
    {
        return strripos($_SERVER['HTTP_USER_AGENT'], 'micromessenger') != false;
    }

    public function is_app_client()
    {
        if (!isset($_SERVER['HTTP_USER_AGENT'])) {
            return false;
        }

        $u = strtolower($_SERVER['HTTP_USER_AGENT']);
        if ($u == null || strlen($u) == 0) {
            return false;
        }

        preg_match('/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/', $u, $res);

        if ($res && count($res) > 0) {
            return true;
        }

        if (strlen($u) < 4) {
            return false;
        }

        preg_match('/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/', substr($u, 0, 4), $res);
        if ($res && count($res) > 0) {
            return true;
        }

        $ipadchar = "/(ipad|ipad2)/i";
        preg_match($ipadchar, $u, $res);
        if ($res && count($res) > 0) {
            return true;
        }

        return false;
    }

    /**
     * Add content to the WC emails.
     *
     * @access public
     * @param WC_Order $order
     * @param bool $sent_to_admin
     * @param bool $plain_text
     */
    public function email_instructions($order, $sent_to_admin, $plain_text = false)
    {
        $method = method_exists($order, 'get_payment_method') ? $order->get_payment_method() : $order->payment_method;
        if ($this->instructions && !$sent_to_admin && $this->id === $method) {
            echo wpautop(wptexturize($this->instructions)) . PHP_EOL;
        }
    }


    public function process_payment($order_id)
    {
        $order = new WC_Order($order_id);
        if (!$order || !$order->needs_payment()) {
            return array(
                'result' => 'success',
                'redirect' => $this->get_return_url($order)
            );
        }
        try {
            if ($this->is_wechat_client()) {
                $result = IOTPay_API::generate_iotpay_order($order, 'WX_JSAPI');
                return array(
                    'result' => 'success',
                    'redirect' => $result->url
                );
            } else {
                return array(
                    'result' => 'success',
                    'redirect' => $order->get_checkout_payment_url(true)
                );
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function jsapi_return()
    {
        $order_id = sanitize_text_field($_GET['order_id']);
        $order = new WC_Order($order_id);
        if ($order && $order->needs_payment()) {
            try {
                $order->payment_complete($order_id);
                header("Location: " . $order->get_checkout_order_received_url());
            } catch (Exception $e) {
                throw $e;
            }
        }
        header("Location: " . home_url() . "/?error=404");
    }

    public function process_refund($order_id, $amount = null, $reason = '')
    {
        return new WP_Error('refuse_error', 'For refund, please log in to iotpay background and use manual refund.');
    }

    public function wc_order_is_paid()
    {
        $order_id = isset($_GET['id']) ? $_GET['id'] : 0;
        if (!$order_id) {
            echo json_encode(array(
                'status' => 'unpaid'
            ));
            exit;
        }

        $order = new WC_Order($order_id);
        if (!$order || $order->needs_payment()) {
            echo json_encode(array(
                'status' => 'unpaid'
            ));
            exit;
        }

        echo json_encode(array(
            'status' => 'paid'
        ));
        exit;
    }

    // IOTPay WeChat Pay -> local
    public function wc_receipt($order_id)
    {
        $order = new WC_Order($order_id);
        if (!$order || !$order->needs_payment()) {
?>
            <script type="text/javascript">
                location.href = '<?php echo $this->get_return_url($order); ?>';
            </script>
            <?php
            return;
        }
        //        $partner_code = $this->get_option('partner_code');
        //        $credential_code = $this->get_option('credential_code');
        try {
            if ($this->is_app_client()) {
                $result = IOTPay_API::generate_iotpay_order($order, 'WX_JSAPI');
                if (!$result->code_url) {
            ?>
                    <ul class="woocommerce-error">
                        <li><?php echo 'There is Something Wrong with getting QR Code, Please Try Again!'; ?></li>
                    </ul><?php
                        }
                            ?>
                <p>Please long press or screen capture to save the QR code image to the album, and then open the image with Wechat to pay.<br>
                    QR code is valid within 5 minutes.</p>

                <div style="padding-top:20px;">
                    <div style="display: inline-block; margin: 0;">
                        <div id="qrcode" style="width:300px;height:300px;display: block;"></div>
                    </div>
                </div>
                <?php
                $url = $result->code_url;
            } else {
                $result = IOTPay_API::generate_iotpay_order($order, 'WX_NATIVE');
                $url = $result->codeUrl;
                if (!$result->codeUrl) {
                ?>
                    <ul class="woocommerce-error">
                        <li><?php echo 'There is Something Wrong with getting QR Code, Please Try Again!'; ?></li>
                    </ul><?php
                        }
                            ?>
                <p>Please scan the QR Code using the Wechat App to complete payment.</p>

                <div style="padding-top:20px;">
                    <div style="display: inline-block; margin: 0;">
                        <div id="qrcode" style="width:300px;height:300px;display: block;"></div>
                        <img style="display: block;" src="<?php echo IOTPAY_URL ?>/assets/images/wechat-scan-note.png" ／>
                    </div>
                    <div style="display: inline-block;  margin: 0;">
                        <img style="display: block;" src="<?php echo IOTPAY_URL ?>/assets/images/wechat-scan.png" ／>
                    </div>
                </div>
            <?php
            }
            ?>
            <script type="text/javascript" src="<?php echo IOTPAY_URL ?>/assets/js/jquery.min.js"></script>
            <script type="text/javascript" src="<?php echo IOTPAY_URL ?>/assets/js/qrcode.min.js"></script>
            <script type="text/javascript">
                new QRCode(document.getElementById("qrcode"), {
                    text: "<?php echo $url; ?>",
                    width: 300,
                    height: 300,
                });
                (function($) {
                    function queryOrderStatus() {
                        $.ajax({
                            type: "GET",
                            url: wc_checkout_params.ajax_url,
                            data: {
                                id: <?php print $order_id ?>,
                                action: 'iotpay_order_status'
                            },
                            timeout: 6000,
                            cache: false,
                            dataType: 'json',
                            success: function(data) {
                                if (data && data.status === "paid") {
                                    location.href = '<?php echo $this->get_return_url($order) ?>';
                                    return;
                                }
                                setTimeout(queryOrderStatus, 2000);
                            },
                            error: function() {
                                setTimeout(queryOrderStatus, 2000);
                            }
                        });
                    }

                    setTimeout(function() {
                        queryOrderStatus();
                    }, 3000);
                })(jQuery);
            </script>
        <?php
        } catch (Exception $e) {
        ?>
            <ul class="woocommerce-error">
                <li><?php echo $e->getMessage(); ?></li>
            </ul><?php
                }
            }
        }
