<?php
/*
 * Plugin Name: IOTPay - WechatPay & Alipay & UnionPay & CreditCard for WooCommerce (加拿大微信,支付宝,银联,信用卡支付)
 * Description: IOTPay - Accept WechatPay & Alipay & UnionPay & CreditCard in WooCommerce, 支持微信、支付宝、银联扫码支付,支持信用卡支付，快捷支付，支付人民币商家收加币
 * Version: 1.3.4
 * Author: IOTPay
 * Author URI:https://www.iotpay.ca/
 * Text Domain: IOTPay - WechatPay & Alipay & UnionPay & CreditCard for WooCommerce
 *
 * WC tested up to: 4.4.1
 *
 */
if (!defined('ABSPATH')) exit(); // Exit if accessed directly

// Include our Gateway Class and register Payment Gateway with WooCommerce
add_action('plugins_loaded', 'iotpay_init', 0);

function iotpay_init()
{
    // If the parent WC_Payment_Gateway class doesn't exist
    // it means WooCommerce is not installed on the site
    // so do nothing
    if (!class_exists('WC_Payment_Gateway')) return;

    define('IOTPAY_FILE', __FILE__);
    define('IOTPAY_URL', rtrim(plugin_dir_url(IOTPAY_FILE), '/'));

    // If we made it this far, then include our Gateway Class
    include_once('includes/class-wc-iotpay-api.php');
    include_once('includes/class-wc-iotpay-gateway-wechatpay.php');
    include_once('includes/class-wc-iotpay-gateway-alipay.php');
    include_once('includes/class-wc-iotpay-gateway-creditcard.php');
    include_once('includes/class-wc-iotpay-gateway-unionpay.php');

    global $IOTPayWechatPay;
    $IOTPayWechatPay = new WC_IOTPay_WechatPay();

    add_action('woocommerce_receipt_' . $IOTPayWechatPay->id, array($IOTPayWechatPay, 'wc_receipt'), 10, 1);
    add_action('woocommerce_api_iotpay_notify', array('IOTPay_API', 'iotpay_notify'));

    global $IOTPayAlipay;
    $IOTPayAlipay = new WC_IOTPay_Alipay();

    // Now that we have successfully included our class,
    // Lets add it too WooCommerce
    add_filter('woocommerce_payment_gateways', 'add_iotpay_gateway');
    function add_iotpay_gateway($methods)
    {
        $methods[] = 'WC_IOTPay_WechatPay';
        $methods[] = 'WC_IOTPay_Alipay';
        $methods[] = 'WC_IOTPay_CreditCard';
        $methods[] = 'WC_IOTPay_UnionPay';
        return $methods;
    }

    // Add custom action links
    add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'iotpay_action_links');
    function iotpay_action_links($links)
    {
        $plugin_links = array(
            '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=iotpay_wechatpay') . '">' . 'Settings' . '</a>',
        );
        // Merge our new link with the default ones
        return array_merge($plugin_links, $links);
    }

    //Show pay type in edit order page for admin.
    add_action('woocommerce_admin_order_data_after_billing_address', 'wc_iotpay_custom_display_admin', 10, 1);
    function wc_iotpay_custom_display_admin($order)
    {
        $method = get_post_meta($order->get_id(), '_payment_method', true);
        if ($method != 'iotpay') {
            return;
        }
        $channel = get_post_meta($order->get_id(), 'channel', true);
        $iotpay_order_id = get_post_meta($order->get_id(), 'mchOrderNo', true);
        echo '<p><strong>' . __('Pay Type') . ': </strong> ' . $channel . '</p>';
        echo '<p><strong>' . __('IOTPay Order Id') . ':</strong> ' . $iotpay_order_id . '</p>';
    }
}
